"""
Deletes an entityType or data from an entityType.
"""
import mcmd.config.config as config
from mcmd import io
from mcmd.client.molgenis_client import delete, delete_data, ensure_resource_exists, ResourceType
from mcmd.io import highlight


# =========
# Arguments
# =========

def arguments(subparsers):
    p_delete = subparsers.add_parser('delete',
                                     help='Delete entities or data',
                                     description="Run 'mcmd delete entity -h' or 'mcmd delete data -h' to view the help"
                                                 " for those sub-commands")
    p_delete.add_argument('--force', '-f',
                          action='store_true',
                          help="Does your delete action without asking if you know it for sure")
    p_delete_subparsers = p_delete.add_subparsers(dest="type")
    p_delete_entity = p_delete_subparsers.add_parser('entity',
                                                     help='Delete an entity(type)')
    p_delete_entity.add_argument('entity_type',
                                 type=str,
                                 metavar='ID',
                                 help="The ID of the entity type you want to delete")
    p_delete_entity.set_defaults(func=delete_entity,
                                 write_to_history=True)
    p_delete_data = p_delete_subparsers.add_parser('data',
                                                   help='Delete data from an entity(type)')
    p_delete_data.add_argument('entity_type',
                               metavar='ID',
                               type=str,
                               help="The ID of the entity type you want to delete all data from")
    p_delete_data.set_defaults(func=delete_all_data,
                               write_to_history=True)


# =======
# Methods
# =======


def _delete_row(entity, row):
    url = '{}{}'.format(config.api('rest2'), entity)
    delete_data(url, [row])


def _delete_all_data(entity):
    io.start('Deleting all data from entity {}'.format(highlight(entity)))
    url = '{}{}'.format(config.api('rest1'), entity)
    delete(url)


def _delete_entity_type(entity):
    io.start('Deleting entity {}'.format(highlight(entity)))
    _delete_row('sys_md_EntityType', entity)


def delete_all_data(args):
    ensure_resource_exists(args.entity_type, ResourceType.ENTITY_TYPE)
    if args.force or (not args.force and io.confirm(
            'Are you sure you want to remove all data from entity: {}?'.format(args.entity_type))):
        _delete_all_data(args.entity_type)


def delete_entity(args):
    ensure_resource_exists(args.entity_type, ResourceType.ENTITY_TYPE)
    if args.force or (not args.force and io.confirm(
            'Are you sure you want to remove the complete entity: {}?'.format(args.entity_type))):
        _delete_entity_type(args.entity_type)



import mcmd.config.config as config
from mcmd import io
from mcmd.client.molgenis_client import ResourceType, post, ensure_resource_exists
from mcmd.io import highlight


# =========
# Arguments
# =========


def arguments(subparsers):
    p_disable = subparsers.add_parser('disable',
                                      help='Disable resources/functionality',
                                      description="Run 'mcmd disable rls -h' to view the help for those sub-commands")
    p_disable_subparsers = p_disable.add_subparsers(dest="type")

    p_disable_rls = p_disable_subparsers.add_parser('rls',
                                                    help='Disables row level security on an entity type')
    p_disable_rls.set_defaults(func=enable_rls,
                               write_to_history=True)
    p_disable_rls.add_argument('entity',
                               type=str,
                               help="The entity type to remove the row level security from")


# =======
# Methods
# =======

def enable_rls(args):
    if not io.confirm('Are you sure you want to disable row level security on %s?' % args.entity):
        return

    io.start('Disabling row level security on entity type %s' % highlight(args.entity))

    ensure_resource_exists(args.entity, ResourceType.ENTITY_TYPE)
    post(config.api('rls'), data={'id': args.entity,
                                  'rlsEnabled': False})



import json
from enum import Enum
from urllib.parse import urljoin

import requests

from mcmd.client import auth
from mcmd.client.request_handler import request
from mcmd.config import config
from mcmd.logging import get_logger
from mcmd.utils.errors import McmdError

log = get_logger()


class ResourceType(Enum):
    ENTITY_TYPE = ('sys_md_EntityType', 'entityclass', 'Entity Type')
    THEME = ('sys_set_StyleSheet', 'stylesheet', 'Stylesheet')
    PACKAGE = ('sys_md_Package', 'package', 'Package')
    PLUGIN = ('sys_Plugin', 'plugin', 'Plugin')

    def get_entity_id(self):
        return self.value[0]

    def get_resource_name(self):
        return self.value[1]

    def get_label(self):
        return self.value[2]

    @classmethod
    def of_label(cls, label):
        return ResourceType[label.replace(' ', '_').upper()]


class PrincipalType(Enum):
    USER = 'user'
    ROLE = 'role'


@request
def get(url):
    return requests.get(url,
                        headers=_get_default_headers())


@request
def grant(principal_type, principal_name, resource_type, identifier, permission):
    data = {'radio-' + identifier: permission}

    if principal_type == PrincipalType.USER:
        data['username'] = principal_name
    elif principal_type == PrincipalType.ROLE:
        data['rolename'] = principal_name.upper()
    else:
        raise McmdError('Unknown principal type: %s' % principal_type)

    url = config.api('perm') + resource_type.get_resource_name() + '/' + principal_type.value
    return requests.post(url,
                         headers={
                             'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                             'x-molgenis-token': auth.get_token()},
                         data=data)


@request
def post(url, data):
    return requests.post(url,
                         headers=_get_default_headers(),
                         data=json.dumps(data))


@request
def post_file(url, file_path, params):
    return requests.post(url,
                         headers={'x-molgenis-token': auth.get_token()},
                         files={'file': open(file_path, 'rb')},
                         params=params)


@request
def post_files(files, url):
    return requests.post(url,
                         headers={'x-molgenis-token': auth.get_token()},
                         files=files)


@request
def delete(url):
    return requests.delete(url,
                           headers=_get_default_headers())


@request
def delete_data(url, data):
    return requests.delete(url,
                           headers=_get_default_headers(),
                           data=json.dumps({"entityIds": data}))


@request
def put(url, data):
    return requests.put(url=url,
                        headers=_get_default_headers(),
                        data=data)


@request
def import_by_url(params):
    return requests.post(config.api('import_url'),
                         headers=_get_default_headers(),
                         params=params)


@request
def get_version():
    return requests.get(urljoin(config.api('rest2'), 'version'),
                        headers={'Content-Type': 'application/json'})


def _get_default_headers():
    headers = {'Content-Type': 'application/json',
               'x-molgenis-token': auth.get_token()}
    return headers


def resource_exists(resource_id, resource_type):
    log.debug('Checking if %s %s exists' % (resource_type.get_label(), resource_id))
    response = get(config.api('rest2') + resource_type.get_entity_id() + '?q=id==' + resource_id)
    return int(response.json()['total']) > 0


def one_resource_exists(resources, resource_type):
    log.debug('Checking if one of [{}] exists in [{}]'.format(','.join(resources), resource_type.get_label()))
    response = get(config.api('rest2') + resource_type.get_entity_id() + '?q=id=in=({})'.format(','.join(resources)))
    return int(response.json()['total']) > 0


def ensure_resource_exists(resource_id, resource_type):
    if not resource_exists(resource_id, resource_type):
        raise McmdError('No %s found with id %s' % (resource_type.get_label(), resource_id))


def ensure_principal_exists(principal_name, principal_type):
    if not principal_exists(principal_name, principal_type):
        raise McmdError('No {} found with id {}'.format(principal_type.value, principal_name))


def principal_exists(principal_name, principal_type):
    if principal_type == PrincipalType.USER:
        return user_exists(principal_name)
    elif principal_type == PrincipalType.ROLE:
        return role_exists(principal_name)
    else:
        raise McmdError('Unknown principal type: %s' % principal_type)


def user_exists(username):
    log.debug('Checking if user %s exists' % username)
    response = get(config.api('rest2') + 'sys_sec_User?q=username==' + username)
    return int(response.json()['total']) > 0


def role_exists(rolename):
    log.debug('Checking if role %s exists' % rolename)
    response = get(config.api('rest2') + 'sys_sec_Role?q=name==' + rolename.upper())
    return int(response.json()['total']) > 0
